@@ -38,13 +38,9 @@ type OutstandingRequest<'r> = Pin<Box<
     dyn Future<Output=Option<Box<[u8]>>> + Send + 'r
     >>;
 
-impl<T> Hcc for T where
-        T: hyper::client::connect::Connect + Clone + Send + Sync + 'static { }
-trait Hcc: hyper::client::connect::Connect + Clone + Send + Sync + 'static { }
-
-struct ClientContext<'c,C> {
+struct ClientContext<'c> {
   ic: &'c InstanceConfig,
-  hclient: &'c Arc<hyper::Client<C>>,
+  hclient: &'c reqwest::Client,
   reporter: &'c parking_lot::Mutex<Reporter<'c>>,
 }
 
@@ -55,8 +51,8 @@ struct TxQueued {
 }
 
 #[throws(AE)]
-fn submit_request<'r, 'c:'r, C:Hcc>(
-  c: &'c ClientContext<C>,
+fn submit_request<'r, 'c:'r>(
+  c: &'c ClientContext,
   req_num: &mut ReqNum,
   reqs: &mut Vec<OutstandingRequest<'r>>,
   upbound: FramesData,
@@ -139,33 +135,41 @@ fn submit_request<'r, 'c:'r, C:Hcc>(
   trace!("{} #{}: req; tx body_len={} frames={}",
          &c.ic, req_num, body_len, upbound.len());
 
-  let body = hyper::body::Body::wrap_stream(
+  let body = http_body_util::StreamBody::new(
     futures::stream::iter(
       content!(
         Bytes,
         into_iter,
         into,
-      ).map(Ok::<Bytes,Void>)
+      ).map(|by| Ok::<_, Void>(http_body::Frame::data(by)))
     )
   );
 
-  let req = hyper::Request::post(&c.ic.url)
-    .header("Content-Type", r#"multipart/form-data; boundary="b""#)
-    .header("Content-Length", body_len)
-    .body(body)
-    .context("construct request")?;
+  let req = {
+    let url = c.ic.url.clone();
+    let mut req = reqwest::Request::new(reqwest::Method::POST, url);
+    let h = req.headers_mut();
+    let ctype = r#"multipart/form-data; boundary="b""#;
+    let ctype = reqwest::header::HeaderValue::from_static(ctype);
+    h.insert("Content-Type", ctype);
+    *req.body_mut() = Some(reqwest::Body::wrap(body));
+    req
+  };
 
-  let resp = c.hclient.request(req);
+  let resp = c.hclient.execute(req);
   let fut = Box::pin(async move {
     let r = async { tokio::time::timeout( c.ic.effective_http_timeout, async {
       let resp = resp.await.context("make request")?;
       let status = resp.status();
-      let mut resp = resp.into_body();
       let max_body = c.ic.max_batch_down.sat() + MAX_OVERHEAD;
+      let body = futures::stream::unfold(resp, |mut resp| async {
+        resp.chunk().await.transpose().map(|r| (r, resp))
+      });
+      pin!(body);
       let resp = read_limited_bytes(
-        max_body, default(), default(), &mut resp
+        max_body, default(), default(), body.as_mut(),
       ).await
-        .discard_data().context("fetching response body")?;
+        .context("fetching response body")?;
 
       if ! status.is_success() {
         throw!(anyhow!("HTTP error status={} body={:?}",
@@ -187,9 +191,9 @@ fn submit_request<'r, 'c:'r, C:Hcc>(
   reqs.push(fut);
 }
 
-async fn run_client<C:Hcc>(
+async fn run_client(
   ic: InstanceConfig,
-  hclient: Arc<hyper::Client<C>>
+  hclient: reqwest::Client,
 ) -> Result<Void, AE>
 {
   debug!("{}: config: {:?}", &ic, &ic);
@@ -291,7 +295,9 @@ async fn run_client<C:Hcc>(
         (got, goti, _) = async { future::select_all(&mut reqs).await },
           if ! reqs.is_empty() =>
         {
-          reqs.swap_remove(goti);
+          // This future was Ready and has returned the value,
+          // which is in `got`.  We don't want the completed future.
+          let _: Pin<Box<dyn Future<Output = _>>> = reqs.swap_remove(goti);
 
           if let Some(got) = got {
             
@@ -338,20 +344,18 @@ async fn main() {
   let opts = <Opts as clap::Parser>::parse();
   let (ics,) = config::startup(
     "hippotat", LinkEnd::Client,
-    &opts.config, &opts.log, |_, ics| 
+    &opts.config, &opts.log, |_, ics|
   {
     PrintConfigOpt(&opts.print_config)
       .implement(ics, )?;
     Ok(())
-  }, |_, ics| {
+  }, |_, ics| async move {
     Ok((ics,))
-  });
+  }).await;
 
-  let https = HttpsConnector::new();
-  let hclient = hyper::Client::builder()
-    .http1_preserve_header_case(true)
-    .build::<_, hyper::Body>(https);
-  let hclient = Arc::new(hclient);
+  let hclient = reqwest::Client::builder()
+    .http1_title_case_headers()
+    .build().expect("build reqwest Client");
 
   info!("starting");
   let () = future::select_all(
