@@ -36,7 +36,7 @@ pub struct InstanceConfig {
   #[client]  pub max_requests_outstanding:     u32,
   #[client]  pub http_retry:                   Duration,
   #[client]  pub success_report_interval:      Duration,
-  #[client]  pub url:                          Uri,
+  #[client]  pub url:                          Url,
   #[client]  pub vroutes:                      Vec<IpNet>,
   #[client]  pub ifname_client:                String,
 
@@ -76,7 +76,7 @@ target_requests_outstanding = 10
 "#;
 
 #[derive(clap::Args,Debug)]
-pub struct Opts {
+pub struct CommonOpts {
   /// Top-level config file or directory
   ///
   /// Look for `main.cfg`, `config.d` and `secrets.d` here.
@@ -86,7 +86,7 @@ pub struct Opts {
   pub config: PathBuf,
   
   /// Additional config files or dirs, which can override the others
-  #[clap(long, multiple=true, number_of_values=1)]
+  #[clap(long, action=clap::ArgAction::Append)]
   pub extra_config: Vec<PathBuf>,
 }
 
@@ -203,7 +203,7 @@ impl_inspectable_config_value!{ ServerName as Display }
 impl_inspectable_config_value!{ ClientName as Display }
 impl_inspectable_config_value!{ u16 as Display }
 impl_inspectable_config_value!{ u32 as Display }
-impl_inspectable_config_value!{ hyper::Uri as Display }
+impl_inspectable_config_value!{ reqwest::Url as Display }
 
 impl_inspectable_config_value!{ IpAddr as Display }
 impl_inspectable_config_value!{ ipnet::IpNet as Display }
@@ -240,7 +240,8 @@ impl Parseable for Secret {
     Secret(s.into())
   }
   #[throws(AE)]
-  fn default() -> Self { Secret(default()) }
+  fn default_for_ordinary() -> Self { Parseable::unspecified() }
+  fn unspecified() -> Self { Secret(default()) }
 }
 impl Debug for Secret {
   #[throws(fmt::Error)]
@@ -278,7 +279,7 @@ impl<'v> RawValRef<'v,'_,'_> {
 }
 
 pub struct Config {
-  pub opts: Opts,
+  pub opts: CommonOpts,
 }
 
 static OUTSIDE_SECTION: &str = "[";
@@ -613,13 +614,28 @@ struct ResolveContext<'c> {
 
 trait Parseable: Sized {
   fn parse(s: Option<&str>) -> Result<Self, AE>;
-  fn default() -> Result<Self, AE> {
+
+  /// Used for lookups with [`ResolveContest::ordinary`] etc.
+  ///
+  /// Fails, if this setting ought to have been specified.
+  /// The caller will add a key name to the error.
+  fn default_for_ordinary() -> Result<Self, AE> {
     Err(anyhow!("setting must be specified"))
   }
-  #[throws(AE)]
-  fn default_for_key(key: &str) -> Self {
-    Self::default().with_context(|| key.to_string())?
-  }
+
+  /// Placeholder (infalliable)
+  ///
+  /// Used (sometimes) for lookups with
+  /// [`ResolveContest::client`],
+  /// [`server`](`ResolveContest::server`) and
+  /// [`computed`](`ResolveContest::server`).
+  ///
+  /// Ie, when the value need not be specified because
+  /// it may not be applicable, or could be computed.
+  ///
+  /// We could use `Default::default` but
+  /// not all the types we want to use implement that.
+  fn unspecified() -> Self;
 }
 
 impl Parseable for Duration {
@@ -628,12 +644,15 @@ impl Parseable for Duration {
     // todo: would be nice to parse with humantime maybe
     Duration::from_secs( s.value()?.parse()? )
   }
+  fn unspecified() -> Duration { Duration::ZERO }
 }
-macro_rules! parseable_from_str { ($t:ty $(, $def:expr)? ) => {
+macro_rules! parseable_from_str { ($t:ty, $def:expr) => {
   impl Parseable for $t {
     #[throws(AE)]
     fn parse(s: Option<&str>) -> $t { s.value()?.parse()? }
-    $( #[throws(AE)] fn default() -> Self { $def } )?
+    #[throws(AE)]
+    fn default_for_ordinary() -> Self { Parseable::unspecified() }
+    fn unspecified() -> Self { $def }
   }
 } }
 parseable_from_str!{u16, default() }
@@ -641,7 +660,12 @@ parseable_from_str!{u32, default() }
 parseable_from_str!{String, default() }
 parseable_from_str!{IpNet, default() }
 parseable_from_str!{IpAddr, Ipv4Addr::UNSPECIFIED.into() }
-parseable_from_str!{Uri, default() }
+
+parseable_from_str!{
+  Url,
+  "hippotat-unspecified:".parse()
+    .expect("failed to parse `hippotat-unspecified:` as a url")
+}
 
 impl<T:Parseable> Parseable for Vec<T> {
   #[throws(AE)]
@@ -652,7 +676,8 @@ impl<T:Parseable> Parseable for Vec<T> {
       .collect::<Result<Vec<_>,_>>()?
   }
   #[throws(AE)]
-  fn default() -> Self { default() }
+  fn default_for_ordinary() -> Self { Parseable::unspecified() }
+  fn unspecified() -> Self { default() }
 }
 
 
@@ -760,7 +785,8 @@ impl<'c> ResolveContext<'c> {
   {
     match self.first_of(key, skl)? {
       Some(y) => y,
-      None => Parseable::default_for_key(key)?,
+      None => Parseable::default_for_ordinary()
+        .with_context(|| key.to_string())?,
     }
   }
 
@@ -779,27 +805,27 @@ impl<'c> ResolveContext<'c> {
 
   #[throws(AE)]
   pub fn client<T>(&self, key: &'static str, skl: SKL) -> T
-  where T: Parseable + Default {
+  where T: Parseable {
     match self.end {
       LinkEnd::Client => self.ordinary(key, skl)?,
-      LinkEnd::Server => default(),
+      LinkEnd::Server => Parseable::unspecified(),
     }
   }
   #[throws(AE)]
   pub fn server<T>(&self, key: &'static str, skl: SKL) -> T
-  where T: Parseable + Default {
+  where T: Parseable {
     match self.end {
       LinkEnd::Server => self.ordinary(key, skl)?,
-      LinkEnd::Client => default(),
+      LinkEnd::Client => Parseable::unspecified(),
     }
   }
 
   #[throws(AE)]
   pub fn computed<T>(&self, _key: &'static str, skl: SKL) -> T
-  where T: Default
+  where T: Parseable
   {
     assert_eq!(skl, SKL::None);
-    default()
+    Parseable::unspecified()
   }
 
   #[throws(AE)]
@@ -859,7 +885,7 @@ impl InstanceConfig {
 
     match end {
       LinkEnd::Client => {
-        if self.url == default::<Uri>() {
+        if self.url == Url::unspecified() {
           let addr = self.addrs.get(0).ok_or_else(
             || anyhow!("client needs addrs or url set")
           )?;
@@ -978,7 +1004,7 @@ impl<'i,T> ResolveGlobal<'i> for T where T: Eq + Clone + Debug + 'i {
 }
 
 #[throws(AE)]
-pub fn read(opts: &Opts, end: LinkEnd)
+pub fn read(opts: &CommonOpts, end: LinkEnd)
             -> (Option<ServerName>, Vec<InstanceConfig>)
 {
   let agg = (||{
@@ -1039,13 +1065,14 @@ pub fn read(opts: &Opts, end: LinkEnd)
   (server_name, ics)
 }
 
-pub fn startup<F,T,G,U>(progname: &str, end: LinkEnd,
-                    opts: &Opts, logopts: &LogOpts,
+pub async fn startup<F,T,G,GFut,U>(progname: &str, end: LinkEnd,
+                    opts: &CommonOpts, logopts: &LogOpts,
                     f: F, g: G) -> U
 where F: FnOnce(Option<ServerName>, &[InstanceConfig]) -> Result<T,AE>,
-      G: FnOnce(T, Vec<InstanceConfig>) -> Result<U,AE>,
+      G: FnOnce(T, Vec<InstanceConfig>) -> GFut,
+      GFut: Future<Output=Result<U,AE>>,
 {
-  (||{
+  async {
     dedup_eyre_setup()?;
     let (server_name, ics) = config::read(opts, end)?;
 
@@ -1053,10 +1080,10 @@ where F: FnOnce(Option<ServerName>, &[InstanceConfig]) -> Result<T,AE>,
     if ics.is_empty() { throw!(anyhow!("no associations, quitting")); }
 
     logopts.log_init()?;
-    let u = g(t, ics)?;
+    let u = g(t, ics).await?;
 
     Ok::<_,AE>(u)
-  })().unwrap_or_else(|e| {
+  }.await.unwrap_or_else(|e| {
     eprintln!("{}: startup error: {}", progname, &e);
     process::exit(8);
   })
